#include <stdio.h>
#include <string.h>

#define FAILED(x) (x<0)

typedef struct CLex {
  FILE *fp;
  int ch;
} CLex ;

static int CLex_Open(CLex *__this, char *filename) {
  if((__this->fp=fopen(filename,"r"))==NULL)
    return -1;
  __this->ch = fgetc(__this->fp);
  return 0;
}

static int CLex_Close(CLex *__this) {
  fclose(__this->fp);
  return 0;
}

static int CLex_SkipLine(CLex *__this) {
  while(__this->ch!='\n'&&__this->ch!=EOF)
    __this->ch=fgetc(__this->fp);
  return 0;
}

static int CLex_SkipSpace(CLex *__this) {
  while(strchr(" \t\n,;",__this->ch)) {
    if (__this->ch==';')
      CLex_SkipLine(__this);
    else
      __this->ch=fgetc(__this->fp);
  }
  return 0;
}

static int CLex_GetNumber(CLex *__this) {
  int ret=-1;
  CLex_SkipSpace(__this);
  while( '0'<=__this->ch && __this->ch <= '9' ) {
    ret = (ret<0?0:ret) * 10 + (__this->ch - '0');
    __this->ch=fgetc(__this->fp); 
  }
  return ret;
}

typedef struct TONE
{
  int tl, fb, ml, kl, ar, dr, sl, rr, am, pm, eg, kr, wf;
} TONE;

static int tone2hex(TONE t[2], int hex[8]) {
  int i;
  for(i=0;i<2;i++) {
    hex[0+i] = (t[i].am<<7) | (t[i].pm<<6) | 
               (t[i].eg<<5) | (t[i].kr<<4) | (t[i].ml);
    hex[2+i] = (t[i].kl<<6);
    hex[4+i] = (t[i].ar<<4) | (t[i].dr<<4);
    hex[6+i] = (t[i].sl<<4) | (t[i].rr<<4);
  }
  hex[2] |= t[0].tl;
  hex[3] |= (t[1].wf<<4)|(t[0].wf<<3)|(t[0].fb);
  return 0;
}

static int txt2tone(char *filename, TONE t[2]) {

  CLex lex;
 
  int i;

  if(FAILED(CLex_Open(&lex,filename)))
    return -1;

  t[0].tl = CLex_GetNumber(&lex);
  if(t[0].tl<0||63<t[0].tl) return -1;
  t[0].fb = CLex_GetNumber(&lex);
  if(t[0].fb<0||7<t[0].fb) return -1;
  for(i=0;i<2;i++) {
    t[i].ar = CLex_GetNumber(&lex);
    if(t[i].ar<0||15<t[i].ar) return -1;
    t[i].dr = CLex_GetNumber(&lex);
    if(t[i].dr<0||15<t[i].dr) return -1;
    t[i].sl = CLex_GetNumber(&lex);
    if(t[i].sl<0||15<t[i].sl) return -1;
    t[i].rr = CLex_GetNumber(&lex);
    if(t[i].rr<0||15<t[i].rr) return -1;
    t[i].kl = CLex_GetNumber(&lex);
     if(t[i].kl<0||3<t[i].kl) return -1;
    t[i].ml = CLex_GetNumber(&lex);
    if(t[i].ml<0||15<t[i].ml) return -1;
    t[i].am = CLex_GetNumber(&lex);
    if(t[i].am<0||1<t[i].am) return -1;
    t[i].pm = CLex_GetNumber(&lex);
    if(t[i].pm<0||1<t[i].pm) return -1;
    t[i].eg = CLex_GetNumber(&lex);
    if(t[i].eg<0||1<t[i].eg) return -1;
    t[i].kr = CLex_GetNumber(&lex);
    if(t[i].kr<0||1<t[i].kr) return -1;
    t[i].wf = CLex_GetNumber(&lex);
    if(t[i].wf<0||1<t[i].wf) return -1;
  }

  CLex_Close(&lex);
  return 0;
}

void help(){
  fprintf(stderr,"Usage: oplltone [options] textfile\n");
  fprintf(stderr,"(Options)\n");
  fprintf(stderr,"  /y : output y command sequence.\n");
}

int main(int argc, char *argv[])
{
  TONE t[2];
  int v[8];
  int i;
  char *input;
  int opt_ycmd=0;

  if(argc<2){
    help();
    return 0;
  }

  for(i=1;i<argc;i++) {
    if(argv[i][0]=='-'||argv[i][0]=='/') {
      switch(argv[i][1]) {
      case 'y':
        opt_ycmd = 1;
        break;
      default:
        help();
        return 1; 
      }
    }
    else input = argv[i];
  }

  memset(&t,-1,sizeof(t[2]));

  if(FAILED(txt2tone(input, t))) {
    fprintf(stderr,"Error.\n");
    printf(";TL FB\n");
    printf(" %02d %02d\n", t[0].tl, t[0].fb);
    printf(";AR DR SL RR KL ML AM PM EG KR WF\n");
    for(i=0;i<2;i++)
      printf(" %2d %2d %2d %2d %2d %2d %2d %2d %2d %2d %2d\n", 
             t[0].ar,t[0].dr,t[0].sl,t[0].rr,t[0].kl,t[0].ml,
             t[0].am,t[0].pm,t[0].eg,t[0].kr,t[0].wf);
  } else {
    tone2hex(t,v);
    for(i=0;i<8;i++)
      if(opt_ycmd)
        printf("y$9010,$%02xy$9030,$%02x\n",i,v[i]);
      else
        printf("$%02x ",v[i]);
    printf("\n"); 
  }

  return 0;
}
